@@ -1,7 +1,7 @@
 package App::MysqlUtils;
 
-our $DATE = '2016-12-15'; # DATE
-our $VERSION = '0.002'; # VERSION
+our $DATE = '2016-12-23'; # DATE
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
@@ -176,10 +176,10 @@ supply `--no-dry-run` or DRY_RUN=0.
 
 Examples:
 
-    # Drop table T1, T2, T3
+    # Drop table T1, T2, T3 (dry-run mode)
     % mysql-drop-tables DB T1 T2 T3
 
-    # Drop all tables with names matching /foo/
+    # Drop all tables with names matching /foo/ (dry-run mode)
     % mysql-drop-tables DB --table-pattern foo
 
     # Actually drop all tables with names matching /foo/, don't delete more than 5 tables
@@ -255,6 +255,67 @@ sub mysql_drop_tables {
     $res->as_struct;
 }
 
+$SPEC{mysql_query} = {
+    v => 1.1,
+    summary => 'Run query and return table result',
+    description => <<'_',
+
+This is like just regular querying, but the result will be returned as table
+data (formattable using different backends). Or, you can output as JSON.
+
+Examples:
+
+    # by default, show as pretty text table, like in interactive mysql client
+    % mysql-query DBNAME "SELECT * FROM t1"
+
+    # show as JSON (array of hashes)
+    % mysql-query DBNAME "QUERY..." --json ;# or, --format json
+
+    # show as CSV
+    % mysql-query DBNAME "QUERY..." --format csv
+
+    # show as CSV table using Text::Table::CSV
+    % FORMAT_PRETTY_TABLE_BACKEND=Text::Table::Org mysql-query DBNAME "QUERY..."
+
+_
+    args => {
+        %args_common,
+        %args_database,
+        query => {
+            schema => 'str*',
+            req => 1,
+            pos => 0,
+            cmdline_src => 'stdin_or_args',
+        },
+        add_row_numbers => {
+            summary => 'Add first field containing number from 1, 2, ...',
+            schema => ['bool*', is=>1],
+        },
+    },
+};
+sub mysql_query {
+    my %args = @_;
+
+    my $dbh = _connect(%args);
+
+    my $sth = $dbh->prepare($args{query});
+    $sth->execute;
+
+    my @columns = @{ $sth->{NAME_lc} };
+    if ($args{add_row_numbers}) {
+        unshift @columns, "_row"; # XXX what if columns contains '_row' already, we need to supply a unique name e.g. '_row2', ...
+    };
+    my @rows;
+    my $i = 0;
+    while (my $row = $sth->fetchrow_hashref) {
+        $i++;
+        $row->{_row} = $i if $args{add_row_numbers};
+        push @rows, $row;
+    }
+
+    [200, "OK", \@rows, {'table.fields'=>\@columns}];
+}
+
 1;
 # ABSTRACT: CLI utilities related to MySQL
 
@@ -270,7 +331,7 @@ App::MysqlUtils - CLI utilities related to MySQL
 
 =head1 VERSION
 
-This document describes version 0.002 of App::MysqlUtils (from Perl distribution App-MysqlUtils), released on 2016-12-15.
+This document describes version 0.003 of App::MysqlUtils (from Perl distribution App-MysqlUtils), released on 2016-12-23.
 
 =head1 SYNOPSIS
 
@@ -342,10 +403,10 @@ supply C<--no-dry-run> or DRY_RUN=0.
 
 Examples:
 
- # Drop table T1, T2, T3
+ # Drop table T1, T2, T3 (dry-run mode)
  % mysql-drop-tables DB T1 T2 T3
  
- # Drop all tables with names matching /foo/
+ # Drop all tables with names matching /foo/ (dry-run mode)
  % mysql-drop-tables DB --table-pattern foo
  
  # Actually drop all tables with names matching /foo/, don't delete more than 5 tables
@@ -401,6 +462,63 @@ that contains extra information.
 
 Return value:  (any)
 
+
+=head2 mysql_query(%args) -> [status, msg, result, meta]
+
+Run query and return table result.
+
+This is like just regular querying, but the result will be returned as table
+data (formattable using different backends). Or, you can output as JSON.
+
+Examples:
+
+ # by default, show as pretty text table, like in interactive mysql client
+ % mysql-query DBNAME "SELECT * FROM t1"
+ 
+ # show as JSON (array of hashes)
+ % mysql-query DBNAME "QUERY..." --json ;# or, --format json
+ 
+ # show as CSV
+ % mysql-query DBNAME "QUERY..." --format csv
+ 
+ # show as CSV table using Text::Table::CSV
+ % FORMAT_PRETTY_TABLE_BACKEND=Text::Table::Org mysql-query DBNAME "QUERY..."
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<add_row_numbers> => I<bool>
+
+Add first field containing number from 1, 2, ...
+
+=item * B<database>* => I<str>
+
+=item * B<host> => I<str> (default: "localhost")
+
+=item * B<password> => I<str>
+
+=item * B<port> => I<int> (default: 3306)
+
+=item * B<query>* => I<str>
+
+=item * B<username> => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-MysqlUtils>.
